//: DYNARRAY.H -- Dynamically-sized array of object pointers
//. Modified to use templates. From "Using C++, 2nd edition"
//. by Bruce Eckel (Osborne/McGraw-Hill 1992)
#ifndef DYNARRAY_H_
#define DYNARRAY_H_

template<class T, int chunk = 10>
class DynArray {
  T ** array;  // starting address of array of T pointers
  int size;  // current size of the array
  int cursor; // to index through the array
  void error(char * msg = "");
public:
  DynArray();
  ~DynArray();
  int add(T *);  // stash an element in the array,
  // increasing the size if necessary.  Returns the index.
  // Both remove() functions return 1 if successful:
  int remove(T *);  // remove an element by pointer
  int remove(int);  // remove an element using its index
  // reset the cursor to the first nonzero element at the
  // top of the array:
  void reset();
  T* next(); // return the pointer to the next element
	     // (empty elements are skipped).  Returns NULL
	     // at end of array.
  int index() { return cursor; } // index of current element
  // return the pointer to the current element:
  T * current() { return array[cursor]; }
  T * operator[](int); // element selection
  int count(); // number of "live" items in the array
};

#pragma option -Jgx  // to generate external references
// must use 
// #pragma option -Jgd
// inside class definition file after inclusion of this file
// so that public definitions for class members are generated.

#endif // DYNARRAY_H_
